@@ -4,12 +4,14 @@ from anadroid.instrument.Types import INSTRUMENTATION_TYPE
 
 
 class BUILD_SYSTEM(Enum):
+    """Enumerates the build systems that can be used to build Android apps."""
     GRADLE = 'Gradle'
     ANT = 'Ant'
     MAVEN = "Maven"
     UNKNOWN = "Unknown"
 
 class TESTING_FRAMEWORK(Enum):
+    """Enumerates supported testing frameworks"""
     MONKEY = "Monkey"
     MONKEY_RUNNER = "Monkeyrunner"
     JUNIT = "JUnit"
@@ -20,7 +22,9 @@ class TESTING_FRAMEWORK(Enum):
     DROIDBOT = "Droidbot"
     OTHER = "Other"
 
+
 class PROFILER(Enum):
+    """Enumerates sota energy profilers."""
     TREPN = 'Trepn'
     GREENSCALER = 'GreenScaler'
     PETRA = 'Petra'
@@ -28,20 +32,27 @@ class PROFILER(Enum):
     MANAFA = "E-manafa"
     NONE = "None"
 
+
 class INSTRUMENTER(Enum):
+    """Enumerates sota instrumentation tools."""
     JINST = 'JInst'
     HUNTER = 'Hunter'
     NONE = "None"
 
+
 class TESTING_APPROACH(Enum):
+    """Enumerates the 3 testing approaches."""
     WHITEBOX = "WhiteBox"
     BLACKBOX = "BlackBox"
     GREYBOX = "GreyBox"
 
+
 class ANALYZER(Enum):
+    """Enumerates the supported analyzers"""
     OLD_ANADROID_ANALYZER = 'Old Anadroid Analyzer'
     MANAFA_ANALYZER = 'Manafa Analyzer'
 
+
 SUPPORTED_TESTING_APPROACHES = {
     TESTING_APPROACH.WHITEBOX
 }
