@@ -2,15 +2,23 @@ from abc import ABC, abstractmethod
 
 
 class AbstractApplication(ABC):
+    """Main class that abstracts an app.
+
+    Attributes:
+        package_name: app package.
+        version: app version.
+
+    """
     def __init__(self, package_name, version=0.0):
         self.package_name = package_name
         self.version = version
         self.on_fg = False
-        super().__init__()
+
+    def get_app_id(self):
+        return f'{self.package_name}_{self.version}'
 
     @abstractmethod
     def start(self):
-        self.on_fg = True
         pass
 
     @abstractmethod
