@@ -3,6 +3,7 @@ import os
 
 
 class ComposedAnalyzer(AbstractAnalyzer):
+    """Provides a way to abstract the usage of a set of AbstractAnalyzers through a single api call."""
     def __init__(self, profiler, inner_analyzers=()):
         super(ComposedAnalyzer, self).__init__(profiler)
         self.inner_analyzers = []
@@ -42,5 +43,5 @@ class ComposedAnalyzer(AbstractAnalyzer):
                 return False
         return True
 
-    def get_val_for_filter(self, filter_name):
-        return super().get_val_for_filter(filter_name)
+    def get_val_for_filter(self, filter_name, add_data=None):
+        return super().get_val_for_filter(filter_name, add_data)
