@@ -13,7 +13,9 @@ def convert_arg(key,val):
     else:
         return "-" + key + " " + val
 
+
 class MonkeyRunnerWorkUnit(WorkUnit):
+    """extends WorkUnit functionality to adapt it to monkeyrunner executions."""
     def __init__(self, bin_cmd):
         super(MonkeyRunnerWorkUnit, self).__init__(bin_cmd)
 
@@ -24,7 +26,7 @@ class MonkeyRunnerWorkUnit(WorkUnit):
         for v in args:
             self.command += " " + v
         print("executing command: " + self.command)
-        execute_shell_command(self.command).validate(Exception("Error running command"))
+        execute_shell_command(self.command).validate(("Error running command"))
 
 
     def config(self, filepath=None, **kwargs):
