@@ -1,23 +1,61 @@
+import re
+from datetime import datetime
 from subprocess import Popen, PIPE, TimeoutExpired
-
-from manafa.utils.Logger import LogSeverity
-from manafa.utils.Logger import log as logm
-
 from textops import find
 import os
+import time
+from enum import Enum
+
+from termcolor import colored
+
+
+class LogSeverity(Enum):
+    SUCCESS = "Success"
+    WARNING = "Warning"
+    INFO = "Info"
+    ERROR = "Error"
+    FATAL = "Fatal"
+
+
+def getColor(sev):
+    return {
+        'Success': 'green',
+        'Warning': 'yellow',
+        'Info': 'blue',
+        'Error': 'magenta',
+        'Fatal': 'red'
+    }.get(sev, 'green')
+
+
+def log(message, log_sev=LogSeverity.INFO, curr_time=None, to_file=True):
+    curr_time = time.time() if curr_time is None else curr_time
+    color = getColor(log_sev.value)
+    adapted_time = re.sub("\s|:", "-", str(datetime.fromtimestamp(curr_time)))
+    str_to_print ="[%s] %s: %s" % (log_sev.value, adapted_time, message)
+    print(colored(str_to_print, color))
+    if to_file:
+        filename = f'{adapted_time}.log'
+        f = open(os.path.join(get_log_dir(), filename), "a+")
+        f.write(str_to_print+"\n")
+        f.close()
+
+
+def get_log_dir():
+    return os.path.join(os.getcwd(), "logs")
+
 
 def get_reference_dir(packname):
     base_dir = os.path.dirname(os.path.dirname(os.path.dirname(__file__)))
-    # base_dir = sysconfig.get_path("purelib")
     return os.path.join(base_dir, packname)
 
+
 def get_resources_dir(packname="anadroid", default_res_dir="resources"):
     ref_dir = get_reference_dir(packname)
     return os.path.join(ref_dir, default_res_dir)
 
 
 def get_keystore_path():
-    return os.path.join(get_resources_dir(),"keys","pynadroid-releases.keystore")
+    return os.path.join(get_resources_dir(), "keys", "pynadroid-releases.keystore")
 
 
 def get_keystore_pwd():
@@ -40,7 +78,7 @@ def get_results_dir(default_results_dir="anadroid_results"):
 
 
 def extract_pkg_name_from_apk(apkpath):
-    res = execute_shell_command("find $ANDROID_HOME/build-tools/ -name \"aapt\" | head -1")
+    res = execute_shell_command("find $ANDROID_HOME/build-tools/ -name \"aapt\" | sort | tail -1")
     res.validate(Exception("Unable to find aapt executable"))
     aapt_executable = res.output.strip()
     res = execute_shell_command(f"{aapt_executable}  dump badging {apkpath} | grep 'package: name='")
@@ -50,13 +88,14 @@ def extract_pkg_name_from_apk(apkpath):
 
 
 def extract_version_from_apk(apkpath):
-    res = execute_shell_command("find $ANDROID_HOME/build-tools/ -name \"aapt\" | head -1")
+    res = execute_shell_command("find $ANDROID_HOME/build-tools/ -name \"aapt\" | sort | tail -1")
     res.validate(Exception("Unable to find aapt executable"))
     aapt_executable = res.output.strip()
     res = execute_shell_command(f"{aapt_executable}  dump badging {apkpath} | grep 'versionName='")
     res.validate(Exception("error while executing aapt"))
     return res.output.split(" ")[3].replace("versionName=", "").replace("'", "")
 
+
 def get_date_str():
     res = execute_shell_command("date +\"%d_%m_%y_%H_%M_%S\"")
     if res.validate(Exception("Unable to get date")):
@@ -69,6 +108,7 @@ def get_apksigner_bin():
         return res.output.split()[0]
     return "$ANDROID_HOME/build-tools/30.0.3/apksigner"
 
+
 def sign_apk(apk_path):
     # deprecated after api 26: "jarsigner -verbose -sigalg SHA2-256withRSA -digestalg SHA2-256  -keystore {keystore} {apk_path} {key_alias} <<< \"{passwd}\"".format(keystore=PYNADROID_KEYSTORE_PATH,apk_path=apk_path,key_alias=KEY_ALIAS,passwd=PASSWORD)
     signer_bin = get_apksigner_bin()
@@ -82,6 +122,7 @@ def sign_apk(apk_path):
     res.validate("error signing apk " + apk_path)
     return res
 
+
 def execute_shell_command(cmd, args=[], timeout=None):
     command = cmd + " " + " ".join(args) if len(args) > 0 else cmd
     out = bytes()
@@ -100,10 +141,11 @@ def execute_shell_command(cmd, args=[], timeout=None):
 
 
 def mega_find(basedir, pattern="*", maxdepth=999, mindepth=0, type_file='n'):
-    basedir_len = len(basedir.split("/"))
-    res = find(basedir, pattern=pattern, only_files=type_file=='f', only_dirs=type_file=='d' )
+    basedir_len = len(basedir.split(os.sep))
+    res = find(basedir, pattern=pattern, only_files=type_file == 'f', only_dirs=type_file == 'd')
     # filter by depth
-    return list(filter(lambda x: basedir_len + mindepth <= len(x.split("/")) <= maxdepth + basedir_len, res))
+    return list(filter(lambda x: basedir_len + mindepth <= len(x.split(os.sep)) <= maxdepth + basedir_len, res))
+
 
 class COMMAND_RESULT(object):
     def __init__(self, res, out, err):
@@ -142,21 +184,21 @@ def log_to_file(content, filename):
         u.write(content + "\n")
 
 
-def logi(message):
-    logm(message, log_sev=LogSeverity.INFO)
+def logi(message, to_file=True):
+    log(message, log_sev=LogSeverity.INFO, to_file=to_file)
 
 
-def logw(message):
-    logm(message, log_sev=LogSeverity.WARNING)
+def logw(message, to_file=True):
+    log(message, log_sev=LogSeverity.WARNING, to_file=to_file)
 
 
-def loge(message):
-    logm(message, log_sev=LogSeverity.ERROR)
+def loge(message, to_file=True):
+    log(message, log_sev=LogSeverity.ERROR, to_file=to_file)
 
 
-def logf(message):
-    logm(message, log_sev=LogSeverity.FATAL)
+def logf(message, to_file=True):
+    log(message, log_sev=LogSeverity.FATAL, to_file=to_file)
 
 
-def logs(message):
-    logm(message, log_sev=LogSeverity.SUCCESS)
\ No newline at end of file
+def logs(message, to_file=True):
+    log(message, log_sev=LogSeverity.SUCCESS, to_file=to_file)
\ No newline at end of file
